#include <iostream>

#define USE_IOCTL

#ifdef USE_IOCTL
#include <termios.h>
#include <sys/ioctl.h>
#else
#include <curses.h>
#include <term.h>
#endif

#include <vector>
#include <unistd.h>

#define DO_INDENTATION

char* helpstring = 
"Program splitenv, version 1.0, 19.08.2008.\n"
"The program is designed for readable printing output of setenv.\n"
"The program receives the information, which it has to reorder,\n" 
"at the standard input stream.\n"
"The typical run of program is: \n"
"shell> setenv | splitenv\n"
"Alternatively one can send the result to a file by \n"
"shell> setenv | splitenv > filename\n"
"The program determines itself to where its output is going.\n"
"In the case of terminal it formats the output according to \n"
"the current number of columns, which it can determine automatically.\n"
"In the second case it \"thinks\" that the number of columns is 10000.\n"
"In both cases the number of colunms can be set manually via an  option.\n"
"Run-time options:\n"
"The program interpret as options all words passed to it.\n"
"-h : Prints help and quit.\n"
"-c : Prints copyright and quit.\n"
"-u : Not split information at \":\" sign like splitpath do this.\n"
"     By default it splits (which is more convenient in general case).\n"
"-l : Indicate that the next argument is the number of columns according\n"
"     to which it should format output.\n"
"-n : Not sort output according to enviromnent variable names and alphabet.\n"
"     Otherwise it will sort it. In practice you might meet cases when this\n"
"     or otherwise that possibility is useful.\n";

char* copyrightstring = 
"Copyright (c) 2008 I. B. Smirnov\n"
"\n"
"This program can be used, copied, modified, and distributed\n"
"according to the terms of GNU General Public License version 3\n"
"published by the Free Software Foundation 29 June 2007,\n"
"and provided that the above copyright notice, this permission notice, \n"
"and notices about any modifications of the original text\n" 
"appear in all copies and in supporting documentation.\n"
"The program is provided \"as is\" without express or implied warranty.\n";




int main(int argc, char *argv[])
{
  int s_splitpath = 1;
  int length_of_line = 10000;
  int s_sort = 1;

  int is_a_term = isatty(STDOUT_FILENO);
  //std::cout<<"is_a_term="<<is_a_term<<'\n';
  if(is_a_term == 1)
  {
#ifdef USE_IOCTL
    winsize ws;
    int iret = ioctl(STDOUT_FILENO, TIOCGWINSZ, &ws);
    if(iret != -1)
    {
      int num = ws.ws_col;
      //std::cout<<"num = "<<num<<'\n';  // for debug
      if(num > 0)
      {
	length_of_line = num;
      }
    }
#else
    initscr();             // initialization of ncurses
    char id[10] = "co";    // command to output number of columns
    int num = tgetnum(id); // output number of columns
    endwin();

    //printf("num=%d\n", num);
    //std::cout<<"num = "<<num<<'\n';  // for debug
    if(num > 0)
    {
      length_of_line = num;
    }
#endif
  }
  if(argc > 1)
  {
    long n;
    for(n=1; n<argc; n++)
    {
      //std::cout<<"n="<<n<<" strlen(argv[n])="<<strlen(argv[n])<<'\n';
      if(argv[n][0] != '-')
      {
	std::cerr<<"splitenv: unclear argument : "<<argv[n]<<'\n';
	exit(1);
      }
      if(strlen(argv[n]) == 2)
      {
	if(argv[n][1] == 'h')
	{
	  std::cout<<helpstring;
	  exit(0);
	}
	if(argv[n][1] == 'c')
	{
	  std::cout<<copyrightstring;
	  exit(0);
	}

	if(argv[n][1] == 'u')
	{ // not split lines at encountering ":" sign like splitpath do it,
	  // otherwise will split them. 
	  s_splitpath = 0;
	}
	else if(argv[n][1] == 'l')
	{   // produce "manually" the length of line (in the next argument)
	  if(n == argc-1)
	  {
	    std::cerr<<"splitenv: l option is given but the length is not given"<<'\n';
	    exit(1);
	  }
	  n++;
	  int il;
	  sscanf(argv[n], "%d", &il);
	  if(il < 50)
	  {
	    std::cerr<<"Unnatural length of line ("<<il<<"), will use 80\n";
	  }
	  else
	  {
	    length_of_line  = il;
	  }
	}
	else if(argv[n][1] == 'n')
	{ // not sort output according to alphabet. 
	  // Otherwise (by default) will sort
          s_sort = 0;
	}
	else if(argv[n][1] == 'h')
	{ // not sort output according to alphabet. 
	  
	}
	else
	{
	  std::cerr<<"splitenv: unknown option :"<<argv[n]<<"\n";
	  exit(1);
	    
	}
      }
      else
      {
	std::cerr<<"splitenv: unknown option :"<<argv[n]<<"\n";
	exit(1);
      }
	
    }
  }

  std::vector< char > input_env;
  char c;
  long max_length = 0; // max length of name of envorinment variable
  // Needs to put = in a single colunm.

  long cur_length = 0;
  int s_name = 1;  // signature that variable name is being read.
  // Otherwise the =-sign or the variable value is read.
  long q = 0;
  long q_names = 0;
  std::cin.get(c);
  while(!std::cin.eof())
  {
    input_env.push_back(c);
    q++;
    if(s_name == 1)
    {
      if(c == '=')
      {  // Found the end of name, make conclusions:
	if(max_length < cur_length) max_length = cur_length;
	s_name = 0;
	cur_length = 0;
      }
      else
      {
	cur_length++;
      }
    }
    else if(c == '\n')
    {
      s_name = 1;
      q_names++;
    }
    std::cin.get(c);
  }
  max_length++;  // to allow one blank for beauty
  //std::cout<<"max_length="<<max_length<<'\n';
  s_name = 1;
  long i_name = 0;
  long n;
  std::vector< std::vector< char > > output_env(q_names);  // vector
  // of output envorinment variables and their values.
  // Needed for further sorting by alphabet.
  long n_name = 0;

  for(n=0; n<q; n++)
  {
    if(s_name == 1)
    {
      //std::cout<<input_env[n]<<' '<<i_name<<'\n';
      if(input_env[n] == '=')
      {
	//std::cout<<i_name<<'\n';
	long m;
	for(m=0; m<max_length - i_name; m++)
	{
	  output_env[n_name].push_back(' ');
	    //std::cout.put(' ');
	}
	output_env[n_name].push_back('=');
	output_env[n_name].push_back(' ');
	//std::cout.put('=');
	//std::cout.put(' ');
	//n+=1;
	s_name = 0;
	i_name = 0;
      }
      else
      {
	output_env[n_name].push_back(input_env[n]);
	//std::cout.put(input_env[n]);
	i_name++;
      }
    }
    else
    {
      output_env[n_name].push_back(input_env[n]);
      //std::cout.put(input_env[n]);
      if(input_env[n] == '\n')
      {
	s_name = 1;
	i_name = 0;
	n_name++;
      }
    }
  }
  /*
  for(n_name=0; n_name< q_names; n_name++)
  {
    long q =  output_env[n_name].size();
    for(n=0; n<q; n++)
    {
      std::cout.put(output_env[n_name][n]);
    }
  }
  */
  for(n_name=0; n_name< q_names; n_name++)
  {
    output_env[n_name].push_back('\0');
  }
  if(s_sort == 1)
  {
    // A primitive sorting (do not currently remember how it is
    // made in stl and lazy to innovate something more adnavced than 
    // the following):
    long n_name1;
    for(n_name=0; n_name< q_names-1; n_name++)
    {
      for(n_name1=n_name+1; n_name1< q_names; n_name1++)
      {
	if(strcmp(&(output_env[n_name][0]), &(output_env[n_name1][0])) > 0)
	{
	  std::vector< char > temp = output_env[n_name];
	  output_env[n_name] = output_env[n_name1];
	  output_env[n_name1] = temp;
	}
      }
    }
  }
  // Output:
  for(n_name=0; n_name< q_names; n_name++)
  {
    long q =  output_env[n_name].size() - 1;  // -1 for the last 0
    /*
    for(n=0; n<q; n++)
    {
      std::cout.put(output_env[n_name][n]);
    }
    */
    long k = 0; // "index of position" needed in order
    // to make indentation of the values to the right side of 
    // "=" column.
    for(n=0; n<q; n++, k++)
    {
#ifdef DO_INDENTATION
      if(s_splitpath == 0)
      {
	if(k > 0  // otherwise would insert indentation blanks 
	   // in the first line
	   // for each variable. 
	   && k % length_of_line == 0
	   && output_env[n_name][n] != '\n' && output_env[n_name][n] != '\0')
	{
	  std::cout.put('\n');
	  long m;
	  for(m=0; m<max_length + 2; m++)
	  {
	    std::cout.put(' ');
	  }
	  k += max_length + 2;
	}
      }
      else
      {
	if(n > 0  // otherwise would insert indentation blanks 
	   // in the first line
	   // for each variable. 
	   && (k % length_of_line == 0 || 
	       output_env[n_name][n-1] == ':')
	   && output_env[n_name][n] != '\n' && output_env[n_name][n] != '\0')
	{
	  std::cout.put('\n');
	  long m;
	  for(m=0; m<max_length + 2; m++)
	  {
	    std::cout.put(' ');
	  }
	  k = max_length + 2;
	}
      }
#endif
      std::cout.put(output_env[n_name][n]);
    }
  }

}
